package xhttp3.internal.connection;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.SocketAddress;
import java.net.SocketException;
import java.net.Proxy.Type;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.NoSuchElementException;
import xhttp3.Address;
import xhttp3.HttpUrl;
import xhttp3.Route;

public final class RouteSelector {
   private final Address address;
   private final RouteDatabase routeDatabase;
   private Proxy lastProxy;
   private InetSocketAddress lastInetSocketAddress;
   private List<Proxy> proxies;
   private int nextProxyIndex;
   private List<InetSocketAddress> inetSocketAddresses;
   private int nextInetSocketAddressIndex;
   private final List<Route> postponedRoutes;

   public RouteSelector(Address address, RouteDatabase routeDatabase) {
      RouteSelector var10000 = this;
      super();
      this.proxies = Collections.emptyList();
      this.inetSocketAddresses = Collections.emptyList();
      this.postponedRoutes = new ArrayList();
      this.address = address;
      this.routeDatabase = routeDatabase;
      HttpUrl var3 = address.url();
      var10000.resetNextProxy(var3, address.proxy());
   }

   private void resetNextProxy(HttpUrl url, Proxy proxy) {
      if (proxy != null) {
         this.proxies = Collections.singletonList(proxy);
      } else {
         this.proxies = new ArrayList();
         List url1;
         if ((url1 = this.address.proxySelector().select(url.uri())) != null) {
            this.proxies.addAll(url1);
         }

         this.proxies.removeAll(Collections.singleton(Proxy.NO_PROXY));
         this.proxies.add(Proxy.NO_PROXY);
      }

      this.nextProxyIndex = 0;
   }

   private boolean hasNextProxy() {
      return this.nextProxyIndex < this.proxies.size();
   }

   private Proxy nextProxy() {
      if (this.hasNextProxy()) {
         List var10000 = this.proxies;
         int var1;
         this.nextProxyIndex = (var1 = this.nextProxyIndex) + 1;
         Proxy var2 = (Proxy)var10000.get(var1);
         this.resetNextInetSocketAddress(var2);
         return var2;
      } else {
         throw new SocketException("No route to " + this.address.url().host() + "; exhausted proxy configurations: " + this.proxies);
      }
   }

   private void resetNextInetSocketAddress(Proxy proxy) {
      this.inetSocketAddresses = new ArrayList();
      String var2;
      int var3;
      if (proxy.type() != Type.DIRECT && proxy.type() != Type.SOCKS) {
         SocketAddress var7;
         if (!((var7 = proxy.address()) instanceof InetSocketAddress)) {
            throw new IllegalArgumentException("Proxy.address() is not an InetSocketAddress: " + var7.getClass());
         }

         InetSocketAddress var10000 = (InetSocketAddress)var7;
         var2 = getHostString((InetSocketAddress)var7);
         var3 = var10000.getPort();
      } else {
         var2 = this.address.url().host();
         var3 = this.address.url().port();
      }

      if (var3 >= 1 && var3 <= 65535) {
         if (proxy.type() == Type.SOCKS) {
            this.inetSocketAddresses.add(InetSocketAddress.createUnresolved(var2, var3));
         } else {
            List proxy1;
            List var9 = proxy1 = this.address.dns().lookup(var2);
            int var8 = 0;

            for(int var4 = var9.size(); var8 < var4; ++var8) {
               InetAddress var5 = (InetAddress)proxy1.get(var8);
               this.inetSocketAddresses.add(new InetSocketAddress(var5, var3));
            }
         }

         this.nextInetSocketAddressIndex = 0;
      } else {
         throw new SocketException("No route to " + var2 + ":" + var3 + "; port is out of range");
      }
   }

   static String getHostString(InetSocketAddress socketAddress) {
      InetAddress var1;
      return (var1 = socketAddress.getAddress()) == null ? socketAddress.getHostName() : var1.getHostAddress();
   }

   private boolean hasNextInetSocketAddress() {
      return this.nextInetSocketAddressIndex < this.inetSocketAddresses.size();
   }

   private InetSocketAddress nextInetSocketAddress() {
      if (this.hasNextInetSocketAddress()) {
         List var10000 = this.inetSocketAddresses;
         int var1;
         this.nextInetSocketAddressIndex = (var1 = this.nextInetSocketAddressIndex) + 1;
         return (InetSocketAddress)var10000.get(var1);
      } else {
         throw new SocketException("No route to " + this.address.url().host() + "; exhausted inet socket addresses: " + this.inetSocketAddresses);
      }
   }

   private boolean hasNextPostponed() {
      return this.postponedRoutes.isEmpty() ^ true;
   }

   private Route nextPostponed() {
      return (Route)this.postponedRoutes.remove(0);
   }

   public boolean hasNext() {
      return this.hasNextInetSocketAddress() || this.hasNextProxy() || this.hasNextPostponed();
   }

   public Route next() {
      if (!this.hasNextInetSocketAddress()) {
         if (!this.hasNextProxy()) {
            if (this.hasNextPostponed()) {
               return this.nextPostponed();
            }

            throw new NoSuchElementException();
         }

         this.lastProxy = this.nextProxy();
      }

      InetSocketAddress var1;
      this.lastInetSocketAddress = var1 = this.nextInetSocketAddress();
      Route var2;
      Route var10001 = var2 = new Route;
      Address var3 = this.address;
      var10001.<init>(var3, this.lastProxy, var1);
      if (this.routeDatabase.shouldPostpone(var2)) {
         this.postponedRoutes.add(var2);
         return this.next();
      } else {
         return var2;
      }
   }

   public void connectFailed(Route failedRoute, IOException failure) {
      if (failedRoute.proxy().type() != Type.DIRECT && this.address.proxySelector() != null) {
         this.address.proxySelector().connectFailed(this.address.url().uri(), failedRoute.proxy().address(), failure);
      }

      this.routeDatabase.failed(failedRoute);
   }
}
